# ./addons/eventscripts/_libs/python/esamlib.py

import base64
import es
import os.path
import StringIO
import sys
import urllib
import xml.etree.ElementTree as ElementTree

import psyco
psyco.full()

"""
Basic Library for interfacing with ESAM
"""

baseurl = 'http://addons.eventscripts.com/'

###

class DBGLog(object):
   def __init__(self):
      self.lines = []

   def write(self, text, dbglvl=None):
      if not dbglvl is None:
         es.dbgmsg(dbglvl, text)

      self.lines.append(text)

   def echo(self):
      for line in self.lines:
         es.dbgmsg(0, line)

dbglog = DBGLog()

###

class ESAMUpload(object):
   """
   Keyword arguments:
   # username - full username.
   # password - password in plain text.
   # basename - basename
   # filename - name of file.
   # filesize - size of file in bytes
   # file_extension - file extension (zip, py, or txt only)
   # version_notes - version notes.
   # version - file version
   # b64_file_content - binary file contents in base64 encoded.
   # addon_name - addon name
   # forum_url - full URL to forum thread (The THREAD not POST (requires ?t=XXXX url)
   # public_variable - public variable for script
   # addon_desc - addon description
   # es_version - minimal ES version required.
   # video_url - full URL to youtube video.
   # addon_summary - 255 character summary for addon.
   # ready4approval - 1/0 flag if addon is ready for approval
   # install_instructions - installation instructions
   # featurelist_url - full featurelist URL (leave blank if none)
   # es_load_args - es_load arguments (ie es_load mugmod or es_load /wcs/myrace/clown
   # status - status of addon in plain text (Hidden or Normal)
   """
   uploadurl = baseurl + 'sdk/upload'
   addurl    = baseurl + 'sdk/add_collaborator'
   removeurl = baseurl + 'sdk/remove_collaborator '

   """ Public functions """

   def addCollaborator(self, username, api_key, basename, collaborator):
      return self.kw_post(self.addurl, username=username, api_key=api_key, basename=basename, collaborator=collaborator)

   def removeCollaborator(self, username, api_key, basename, collaborator):
      return self.kw_post(self.removeurl, username=username, api_key=api_key, basename=basename, collaborator=collaborator)

   def upload(self, username, api_key, basename, path, addon_name, version, version_notes, addon_desc,
    addon_summary, es_load_args, es_version, forum_url='', public_variable='', video_url='', install_instructions='',
    featurelist_url='', ready4approval=False, status='Normal'):

      filesize = os.path.getsize(path)
      filename = path.replace('\\', '/').rsplit('/', 1)[1]
      file_extension = filename.rsplit('.', 1)[1]

      f = open(path, 'rb')
      b64_file_content = base64.encodestring(f.read())
      f.close()

      return self.kw_post(self.uploadurl, username=username, api_key=api_key, basename=basename, filename=filename, filesize=filesize,
       file_extension=file_extension, version_notes=version_notes, version=version, b64_file_content=b64_file_content,
       addon_name=addon_name, forum_url=forum_url, public_variable=public_variable, addon_desc=addon_desc,
       es_version=es_version, video_url=video_url, addon_summary=addon_summary, ready4approval=int(ready4approval),
       install_instructions=install_instructions, featurelist_url=featurelist_url, es_load_args=es_load_args, status=status)

   """ Private functions """

   def kw_post(self, *a, **kw):
      if len(a) <> 1: raise TypeError, 'kw_post takes exactly 1 argument (%s given)' % len(a)
      u = urllib.urlopen(a[0], data=urllib.urlencode(kw))
      d = u.read()
      u.close()

      return d

esamupload = ESAMUpload()

def addCollaborator(*a, **kw):
   return esamupload.addCollaborator(*a, **kw)

def removeCollaborator(*a, **kw):
   return esamupload.removeCollaborator(*a, **kw)

def upload(*a, **kw):
   return esamupload.upload(*a, **kw)

"""
# API keys can be obtained from: http://addons.eventscripts.com/manage/api_key/

# Upload to in-use basename gungame3
>>> esamlib.upload('SuperDave', my_api_key, 'gungame3', 'c:\\Redirect.zip', 'Redirect', '1.0', 'Version notes', 'Another SD test upload', 'Test summary', 'redirect', '2.0')
'Error: Basename in use by another user.'
 
# Upload to available basename sd_redirect
>>> esamlib.upload('SuperDave', my_api_key, 'sd_redirect', 'c:\\Redirect.zip', 'Redirect', '1.0', 'Version notes', 'Another SD test upload', 'Test summary', 'redirect', '2.0')
'Success! Addon ID# 16569'

# Add GODJonez as a collaborator on the redirect project
>>> esamlib.addCollaborator('SuperDave', my_api_key, 'sd_redirect', 'GODJonez')
'GODJonez added successfully to sd_redirect'

# Remove GODJonez as a collaborator on the redirect project
>>> esamlib.removeCollaborator('SuperDave', my_api_key, 'sd_redirect', 'GODJonez')
'GODJonez removed successfully from sd_redirect'
"""

###

# Search type constants
SEARCH_ADDON       = 'addon' # addon - Searchs multiple fields within an addon. (name, desc, etc)
SEARCH_DESCRIPTION = 'desc'  # desc  - Addon description
SEARCH_ESVERSION   = 'esmin' # esmin - All addons by minimum ES version 
SEARCH_NAME        = 'name'  # name  - Addon Name
SEARCH_PUBLICVAR   = 'pvar'  # pvar  - All addons by a public variable
SEARCH_USER        = 'user'  # user  - All addons by a user

# Top query constance
TOP_ADDONCOUNT = 'addons' # Unavailable for topAddons
TOP_DOWNLOADS  = 'downloads'
TOP_WOOTS      = 'woots'

class ESAMQuery(object):
   d_counturl   = baseurl + 'sdk/downloadcount/'
   getcollaburl = baseurl + 'sdk/collaborators/'
   searchurl    = baseurl + 'sdk/search/'
   topuserurl   = baseurl + 'sdk/topuser'
   topaddonsurl = baseurl + 'sdk/topaddon'
   updateurl    = baseurl + 'sdk/updatedsince/'
   watchedurl   = baseurl + 'sdk/watched_addons/'
   

   """ Public functions """

   def downloadCount(self, basename):
      u = urllib.urlopen(self.d_counturl + basename)
      data = u.read()
      u.close()
      return int(data)

   def getCollaborators(self, basename):
      u = urllib.urlopen(self.getcollaburl + basename)
      data = ElementTree.XML(u.read())
      u.close()
      return data.findtext('leader'), [x.text for x in data.getiterator('collaborator')]

   def search(self, searchtype, text):
      return self.getXMLList(self.searchurl + text + '/' + searchtype, 'addon')

   def topAddons(self, sortby, count=10):
      if sortby == TOP_ADDONCOUNT: raise ValueError, 'TOP_ADDONCOUNT is not a valid sort for topAddons'
      return self.getXMLList(self.topaddonsurl + sortby + '/%i' % count, 'addon')

   def topUsers(self, sortby, count=10):
      return self.getXMLList(self.topuserurl + sortby + '/%i' % count, 'user')

   def updated_since(self, timestamp):
      return self.getXMLList(self.updateurl + '%i' % timestamp, 'addon')

   def watchedBy(self, username):
      u = urllib.urlopen(self.watchedurl + base64.b64encode(username))
      data = ElementTree.XML(u.read())
      u.close()
      return self.spliceBy(data.find('watched'), 'addon')

   """ Data-manipulation functions """

   def getXMLList(self, url, searchname):
      u = urllib.urlopen(url)
      data = ElementTree.XML(u.read())
      u.close()
      return self.spliceBy(data, searchname)

   def spliceBy(self, data, searchname):
      result = []
      for item in data.findall(searchname):
         item_data = {}
         for subitem in item.getiterator():
            name = subitem.tag
            value = item.findtext(name)
            if value:
               item_data[name] = value

         if item_data:
            result.append(item_data)

      return result

esamquery = ESAMQuery()

def downloadCount(*a, **kw):
   return esamquery.downloadCount(*a, **kw)

def getCollaborators(*a, **kw):
   return esamquery.getCollaborators(*a, **kw)

def search(*a, **kw):
   return esamquery.search(*a, **kw)

def topAddons(*a, **kw):
   return esamquery.topAddons(*a, **kw)

def topUsers(*a, **kw):
   return esamquery.topUsers(*a, **kw)

def updated_since(*a, **kw):
   return esamquery.updated_since(*a, **kw)

def watchedBy(*a, **kw):
   return esamquery.watchedBy(*a, **kw)

"""
# Unless otherwise specified, query functions return a list (in order, if meaningful) of dictionaries of addon/user information.

# Query the leader and collaborators of an addon with the return format (leader, [collaborators,])
>>> esamlib.getCollaborators('sentrysounds')
('Mattie', ['HellenAngel'])

# Query [NATO]Hunter's addons
>>> result = esamlib.search(esamlib.SEARCH_USER, 'Hunter')

# Query all addons that require at least ES 2.1
>>> result = esamlib.search(esamlib.SEARCH_ESVERSION, '2.1')

# Query the top four addons by woots
>>> result = esamlib.topAddons(esamlib.TOP_WOOTS, 4)

# Query the top three users by number of addons
>>> result = esamlib.topUsers(esamlib.TOP_ADDONCOUNT, 3)

# Query the top ten users by downloads
>>> result = esamlib.topUsers(esamlib.TOP_DOWNLOADS)

# Query the addons updated less than one day ago
>>> result = esamlib.updated_since(time.time() - 86400)

# Query addons watched by XE_ManUp
>>> result = esamlib.watchedBy('XE_ManUp')
"""

###

import installlib # Don't move this

class ESAMAddon(installlib.Addon):
   """
      # A farily light "wrapper" around the ESAM SDK
        Also provides basic download method
   """
   baseurl = baseurl

   def __init__(self, xmldata, log=dbglog):
      self.data = xmldata
      self.log  = log

   def __getattr__(self, name):
      """
         # Get some addon attributes
            - id
            - author
            - authorid
            - name
            - basename
            - approved
            - es_load
            - date
            - woots
            - desc
            - summary
            - currentversion
            - versionnotes
            - video
            - defaultssid
      """
      return_val = self.get(name)

      if return_val is None:
         raise AttributeError, 'Addon %s has no \'%s\' attribute' % (self.basename, name)

      return return_val

   def download(self):
      """
         # Returns a file instance of the addons latest downloadable version
      """
      if self.approved <> '1': return None

      try:
         u    = urllib.urlopen(self.baseurl + 'addons/download/' + self.basename)
         addon = StringIO.StringIO(u.read())
         u.close()

         return addon

      except IOError:
         info = sys.exc_info()
         self.log.write('ESAMLib: %s - %s' % (info[0].__name__, info[1]), 2)

      return None

   def get(self, attribute):
      return self.data.findtext(attribute) if self.data else None

   def getElements(self):
      return self.data.getiterator() if self.data else []

   def getElementNames(self):
      return map(lambda x: x.tag, self.getElements())


def getAddon(basename, log=dbglog):
   try:
      u   = urllib.urlopen(baseurl + 'sdk/addoninfo/' + basename)
      xml = ElementTree.XML(u.read()).find('addon')
      u.close()

      return ESAMAddon(xml, log) if xml else None

   except IOError:
      info = sys.exc_info()
      log.write('ESAMLib: %s - %s' % (info[0].__name__, info[1]), 2)

   return None

###

class ESAMUser(object):
   class UserAddons(object):
      """
         # Class to deal with the users addons
      """
      def __init__(self, data, log=dbglog):
         self.data = data
         self.log  = log

         self.addons = {}

      def __getattr__(self, name):
         """
            # Provides a quick 'n' easy way to get addon attributes
            # Uses the basename and self.find to return an addon class
         """
         return_val = self.find(name)

         if return_val is None:
            raise AttributeError, 'UserAddons instance has no \'%s\' attribute' % name

         return return_val

      def find(self, addonname):
         """
            # Finds and returns an addon by its basename
         """
         if addonname in self.addons: return self.addons[addonname]

         for addon in self.getAddonList():
            if addonname == addon.findtext('basename'):
               return_addon = self.addons[addonname] = ESAMAddon(addon, self.log)
               return return_addon

         return None

      def isAddon(self, basename):
         """
            # Returns true if the addon exists
         """
         return basename in self.getAddonNameList()

      def getAddonList(self):
         return self.data.findall('addon')

      def getAddonNameList(self):
         return map(lambda x: x.findtext('basename'), self.getAddonList())

   """
      # Begin User class
      # A farily light "wrapper" around the ESAM SDK
        Also provides basic download method
   """

   def __init__(self, xmldata, addondata, log=dbglog):
      self.xmldata = xmldata
      self.addons  = self.UserAddons(addondata, log)

   def __getattr__(self, name):
      """
         # Allows you to get all the user data from the XML
         # The following attributes are available
            - username
            - userid
            - addontotal
            - addonrank
            - woottotal
            - wootrank
            - downloadtotal
            - downloadrank
      """
      return_val = self.get(name)

      if return_val is None:
         raise AttributeError, 'User instance with has no \'%s\' attribute' % name

      return return_val

   def get(self, attribute):
      return self.xmldata.findtext(attribute) if self.xmldata else None

   def getElements(self):
      return self.xmldata.getiterator() if self.xmldata else []

   def getElementNames(self):
      return filter(self.get, map(lambda x: x.tag, self.getElements()))


def getUser(username, log=dbglog):
   try:
      u   = urllib.urlopen(baseurl + 'sdk/getuserid/' + base64.b64encode(username))
      tid = u.read()
      u.close()

      if tid == "false" or not tid: return None

      u      = urllib.urlopen(baseurl + 'sdk/getuserinfo/' + tid)
      xmldata = ElementTree.XML(u.read())
      u.close()
      
      u       = urllib.urlopen(baseurl + 'sdk/getuseraddons/' + tid)
      addondata = ElementTree.XML(u.read())
      u.close()

      return ESAMUser(xmldata, addondata, log)

   except IOError:
      info = sys.exc_info()
      log.write('ESAMLib: %s - %s' % (info[0].__name__, info[1]), 2)

   return None

"""
>>> import esamlib

>>> sd = esamlib.getUser('SuperDave')

>>> sd.getElementNames()
['username', 'userid', 'addontotal', 'addonrank', 'woottotal', 'wootrank', 'downloadtotal', 'downloadrank']

>>> sd.username
'SuperDave'
>>> sd.userid
'11091'
>>> sd.downloadrank
'4'

>> sd.get('userid')
'11091'

>>> sd.addons.getAddonNameList()
['cheapbombtimer', 'regenmod', 'cheapzombie', 'cheapdeathmatch', 'gunmod', 'cheapwirecut', 'swapwithvictim', 'slomomod', 'slobot', 'knifegrenade', 'newrewards', 'cssoundtrack', 'cheapbleed', 'explosivedeaths', 'nadeslay', 'healmates', 'teamfog', 'healthregen', 'weaponstats', 'splashdamage', 'doublevip', 'cheapbeacon','nowyouseeme', 'coloredhealth', 'bomberman', 'sourceradar', 'ezrestrict', 'assistedkills', 'randmaps', 'iedmod', 'safespawn', 'ezfilter', 'extendedevents', 'iptocountry', 'cheapbulletwhizz', 'cheapemote']

>>> sd.addons.cssoundtrack
<esamlib.Addon object at 0x1173AB70>
>>> sd.addons.find('cssoundtrack')
<esamlib.Addon object at 0x1173AB70>

>>> sd.addons.cssoundtrack.getElementNames()
['addon', 'id', 'author', 'authorid', 'name', 'basename', 'approved', 'es_load', 'date', 'woots', 'desc', 'summary', 'currentversion', 'versionnotes', 'video','defaultssid']

>>> sd.addons.cssoundtrack.id
'13758'
>>> sd.addons.cssoundtrack.currentversion
'7'
>>> sd.addons.cssoundtrack.video
'http://www.youtube.com/v/G3S8l3BB4J0'
>>> sd.addons.cssoundtrack.woots
'21'
>>> sd.addons.cssoundtrack.date
'2007-05-02 04:21:26'
>>> sd.addons.cssoundtrack.defaultssid
'200'
"""

"""
>>> import esamlib

>>> css = esamlib.getAddon('cssoundtrack')

>>> css.getElementNames()
['addon', 'id', 'author', 'authorid', 'name', 'basename', 'approved', 'es_load', 'date', 'woots', 'desc', 'summary', 'currentversion', 'versionnotes', 'video','defaultssid']

>>> css.downloadAddon()
<StringIO.StringIO instance at 0x115BF738>

>>> css.currentversion
'7'
>>> css.approved
'1'

>>> css.get('approved')
'1'
"""